// drawdoc.cpp : implementation of the CDrawDoc class
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1998 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.

#include "stdafx.h"
#include "drawcli.h"

#include "drawdoc.h"
#include "drawvw.h"
#include "drawobj.h"
#include "cntritem.h"
#include "summpage.h"
#include "statpage.h"

#include "SrvrItem.h"

#include "mainfrm.h"


#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif



/////////////////////////////////////////////////////////////////////////////
// CDrawDoc

IMPLEMENT_DYNCREATE(CDrawDoc, COleServerDoc)

BEGIN_MESSAGE_MAP(CDrawDoc, COleServerDoc)
	//{{AFX_MSG_MAP(CDrawDoc)
	ON_COMMAND(ID_VIEW_PAPERCOLOR, OnViewPaperColor)
	ON_COMMAND(ID_FILE_SUMMARYINFO, OnFileSummaryInfo)
	//}}AFX_MSG_MAP
	// Enable default OLE container implementation
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, COleServerDoc::OnUpdatePasteMenu)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE_LINK, COleServerDoc::OnUpdatePasteLinkMenu)
	ON_UPDATE_COMMAND_UI(ID_OLE_EDIT_LINKS, COleServerDoc::OnUpdateEditLinksMenu)
	ON_COMMAND(ID_OLE_EDIT_LINKS, COleServerDoc::OnEditLinks)
	ON_UPDATE_COMMAND_UI(ID_OLE_VERB_FIRST, COleServerDoc::OnUpdateObjectVerbMenu)
		// MAPI support
	ON_COMMAND(ID_FILE_SEND_MAIL, OnFileSendMail)
	ON_UPDATE_COMMAND_UI(ID_FILE_SEND_MAIL, OnUpdateFileSendMail)
END_MESSAGE_MAP()


//HLink begin
BEGIN_INTERFACE_MAP(CDrawDoc, COleServerDoc)
	INTERFACE_PART(CDrawDoc, IID_IHlinkTarget, HlinkTarget)
	INTERFACE_PART(CDrawDoc, IID_IHlinkSite, HlinkSite)
END_INTERFACE_MAP()
//HLink end


/////////////////////////////////////////////////////////////////////////////
// CDrawDoc construction/destruction

CDrawDoc::CDrawDoc()
{
	EnableCompoundFile();

	m_nMapMode = MM_ANISOTROPIC;
	m_paperColor = RGB(255, 255, 255);
	m_pSummInfo = NULL;
	ComputePageSize();

	m_pLastActivatedView=NULL; //Bookmarks

	m_dwBrowseContextRegister=0; //HLink
	m_bHidePending=FALSE; //HLink
}

CDrawDoc::~CDrawDoc()
{
	POSITION pos = m_objects.GetHeadPosition();
	while (pos != NULL)
		delete m_objects.GetNext(pos);
	delete m_pSummInfo;
}

BOOL CDrawDoc::OnNewDocument()
{
	if (!COleServerDoc::OnNewDocument())
		return FALSE;

	// reinitialization code
	// (SDI documents will reuse this document)
	if(m_pSummInfo != NULL)
		delete m_pSummInfo;
	m_pSummInfo = new CSummInfo;
	// Title, Subject, Author, Keywords default to empty string
	// Comments, Template, SavedBy default to empty string
	// LastSave, LastPrint, EditTime, RevNum default to 0
	m_pSummInfo->StartEditTimeCount();
	m_pSummInfo->RecordCreateDate();
	m_pSummInfo->SetNumPages(1);
	// NumWords, NumChars default to 0
	m_pSummInfo->SetAppname( _T("DrawCli") );
	// Security defaults to 0
	return TRUE;
}

BOOL CDrawDoc::OnOpenDocument(LPCTSTR lpszPathName)
{
	USES_CONVERSION;

	if( m_pSummInfo != NULL)
		delete m_pSummInfo;
	m_pSummInfo = new CSummInfo;
	m_pSummInfo->StartEditTimeCount();

	return COleServerDoc::OnOpenDocument(lpszPathName);
}

BOOL CDrawDoc::OnSaveDocument(LPCTSTR lpszPathName)
{
	USES_CONVERSION;

	m_pSummInfo->RecordSaveDate();
	m_pSummInfo->IncrRevNum();
	m_pSummInfo->SetLastAuthor(m_pSummInfo->GetAuthor());
	m_pSummInfo->AddCountToEditTime();
	m_pSummInfo->StartEditTimeCount();
	return COleServerDoc::OnSaveDocument(lpszPathName);
}

//Hlink begin
void CDrawDoc::OnCloseDocument()
{
	POSITION pos=m_objects.GetHeadPosition();
	while(pos)
	{
		CDrawObj* pObj=m_objects.GetNext(pos);
		if(pObj->IsKindOf(RUNTIME_CLASS(CDrawHLink)))
		{
			CDrawHLink* pHlinkObj=(CDrawHLink*)pObj;
			if(pHlinkObj->m_pHlink!=NULL)
				pHlinkObj->m_pHlink=NULL;
		}
	}
	
	if(m_pIHlinkBrowseContext)
	{
		m_pIHlinkBrowseContext->Revoke(m_dwBrowseContextRegister);
		m_pIHlinkBrowseContext=NULL;
	}

	COleServerDoc::OnCloseDocument();
}

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc serialization

void CDrawDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		m_pSummInfo->WriteToStorage(m_lpRootStg);

		//Bookmarks begin
		int nSize=m_arrayBookmarks.GetSize();
		ar << nSize;
		for(int i=0; i<nSize; i++)
			m_arrayBookmarks[i].Serialize(ar);
		//Bookmarks end

		ar << m_paperColor;
		m_objects.Serialize(ar);
	}
	else
	{
		m_pSummInfo->ReadFromStorage(m_lpRootStg);

		//Bookmarks begin
		int nSize=0;
		ar >> nSize;
		for(int i=0; i<nSize; i++)
		{
			Bookmark bm;
			bm.Serialize(ar);
			m_arrayBookmarks.Add(bm);
		}
		//Bookmarks end

		ar >> m_paperColor;
		m_objects.Serialize(ar);
	}

	// By calling the base class COleServerDoc, we enable serialization
	//  of the container document's COleClientItem objects automatically.
	COleServerDoc::Serialize(ar);

}


/////////////////////////////////////////////////////////////////////////////
// CDrawDoc implementation

void CDrawDoc::Draw(CDC* pDC, CDrawView* pView)
{
	POSITION pos = m_objects.GetHeadPosition();
	while (pos != NULL)
	{
		CDrawObj* pObj = m_objects.GetNext(pos);
		pObj->Draw(pDC);
		if (pView->m_bActive && !pDC->IsPrinting() && pView->IsSelected(pObj) && pView->m_bDesignMode) //Design
			pObj->DrawTracker(pDC, CDrawObj::selected);
	}
}

void CDrawDoc::Add(CDrawObj* pObj)
{
	m_objects.AddTail(pObj);
	pObj->m_pDocument = this;
	SetModifiedFlag();
}

void CDrawDoc::Remove(CDrawObj* pObj)
{
	// Find and remove from document
	POSITION pos = m_objects.Find(pObj);
	if (pos != NULL)
		m_objects.RemoveAt(pos);
	// set document modified flag
	SetModifiedFlag();

	// call remove for each view so that the view can remove from m_selection
	pos = GetFirstViewPosition();
	while (pos != NULL)
		((CDrawView*)GetNextView(pos))->Remove(pObj);
}

// point is in logical coordinates
CDrawObj* CDrawDoc::ObjectAt(const CPoint& point)
{
	CRect rect(point, CSize(1, 1));
	POSITION pos = m_objects.GetTailPosition();
	while (pos != NULL)
	{
		CDrawObj* pObj = m_objects.GetPrev(pos);
		if (pObj->Intersects(rect))
			return pObj;
	}

	return NULL;
}

void CDrawDoc::ComputePageSize()
{
	CSize new_size(850, 1100);  // 8.5" x 11" default

	CPrintDialog dlg(FALSE);
	if (AfxGetApp()->GetPrinterDeviceDefaults(&dlg.m_pd))
	{
		// GetPrinterDC returns a HDC so attach it
		CDC dc;
		HDC hDC= dlg.CreatePrinterDC();
		ASSERT(hDC != NULL);
		dc.Attach(hDC);

		// Get the size of the page in loenglish
		new_size.cx = MulDiv(dc.GetDeviceCaps(HORZSIZE), 1000, 254);
		new_size.cy = MulDiv(dc.GetDeviceCaps(VERTSIZE), 1000, 254);
	}

	// if size changed then iterate over views and reset
	if (new_size != m_size)
	{
		m_size = new_size;
		POSITION pos = GetFirstViewPosition();
		while (pos != NULL)
			((CDrawView*)GetNextView(pos))->SetPageSize(m_size);
	}
}

void CDrawDoc::OnViewPaperColor()
{
	CColorDialog dlg;
	if (dlg.DoModal() != IDOK)
		return;

	m_paperColor = dlg.GetColor();
	SetModifiedFlag();
	UpdateAllViews(NULL);
}

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc diagnostics

#ifdef _DEBUG
void CDrawDoc::AssertValid() const
{
	COleServerDoc::AssertValid();
}

void CDrawDoc::Dump(CDumpContext& dc) const
{
	COleServerDoc::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc commands

void CDrawDoc::OnFileSummaryInfo()
{
	ASSERT_VALID(this);

	CPropertySheet sheet( _T("Document Properties") );
	CSummPage summ;
	CStatPage stat;
	sheet.AddPage( &summ );
	sheet.AddPage( &stat );

	summ.m_strAppname = m_pSummInfo->GetAppname();
	summ.m_strTitle   = m_pSummInfo->GetTitle();
	summ.m_strSubj    = m_pSummInfo->GetSubject();
	summ.m_strAuthor  = m_pSummInfo->GetAuthor();
	summ.m_strKeywd   = m_pSummInfo->GetKeywords();
	summ.m_strCmt     = m_pSummInfo->GetComments();
	summ.m_strTempl   = m_pSummInfo->GetTemplate();

	stat.m_strSavedBy    = m_pSummInfo->GetLastAuthor();
	stat.m_strRevNum     = m_pSummInfo->GetRevNum();
	stat.m_strEditTime   = m_pSummInfo->GetEditTime();
	stat.m_strLastPrint  = m_pSummInfo->GetLastPrintDate();
	stat.m_strCreateDate = m_pSummInfo->GetCreateDate();
	stat.m_strLastSave   = m_pSummInfo->GetLastSaveDate();
	stat.m_strNumPages   = m_pSummInfo->GetNumPages();
	stat.m_strNumWords   = m_pSummInfo->GetNumWords();
	stat.m_strNumChars   = m_pSummInfo->GetNumChars();
	stat.m_strSecurity   = m_pSummInfo->GetSecurity();

	if (sheet.DoModal() != IDOK)
		return;

	m_pSummInfo->SetAuthor(summ.m_strAuthor);
	m_pSummInfo->SetKeywords(summ.m_strKeywd);
	m_pSummInfo->SetSubject(summ.m_strSubj);
	m_pSummInfo->SetComments(summ.m_strCmt);
	m_pSummInfo->SetTemplate(summ.m_strTempl);
	m_pSummInfo->SetTitle(summ.m_strTitle);

	SetModifiedFlag();
}


//Doc server begin
/////////////////////////////////////////////////////////////////////////////
// CDrawDoc server implementation

COleServerItem* CDrawDoc::OnGetEmbeddedItem()
{
	// OnGetEmbeddedItem is called by the framework to get the COleServerItem
	//  that is associated with the document.  It is only called when necessary.

	CDrawSrvrItem* pItem = new CDrawSrvrItem(this);
	ASSERT_VALID(pItem);
	return pItem;
}

/////////////////////////////////////////////////////////////////////////////
// CDrawDoc Active Document server implementation

CDocObjectServer *CDrawDoc::GetDocObjectServer(LPOLEDOCUMENTSITE pDocSite)
{
	return new CDocObjectServer(this, pDocSite);
}
//Doc server end


//Bookmarks begin
CDrawDoc::Bookmark::Bookmark()
{
	m_bHidden=FALSE;
}

CDrawDoc::Bookmark::Bookmark(const CDrawDoc::Bookmark& other)
{
	this->m_strName=other.m_strName;
	this->m_ptBookmark=other.m_ptBookmark;
	this->m_bHidden=other.m_bHidden;
}

CDrawDoc::Bookmark& CDrawDoc::Bookmark::operator=(const CDrawDoc::Bookmark& other)
{
	this->m_strName=other.m_strName;
	this->m_ptBookmark=other.m_ptBookmark;
	this->m_bHidden=other.m_bHidden;
	return *this;
}

void CDrawDoc::Bookmark::Serialize(CArchive& ar)
{
	if(ar.IsStoring())
	{
		ar << m_strName;
		ar << m_ptBookmark;
		ar << m_bHidden;
	}
	else
	{
		ar >> m_strName;
		ar >> m_ptBookmark;
		ar >> m_bHidden;
	}
}




BOOL CDrawDoc::CreateBookmark(LPCTSTR lpstrBookmarkName,CPoint ptDoc,BOOL bHidden)
{
	int nPos=IndexOfBookmark(lpstrBookmarkName);
	if(nPos==-1)
	{
		Bookmark bm;
		bm.m_strName=lpstrBookmarkName;
		bm.m_ptBookmark=ptDoc;
		bm.m_bHidden=bHidden;
		m_arrayBookmarks.Add(bm);
		SetModifiedFlag();
		return TRUE;
	}

	return FALSE;
}


BOOL CDrawDoc::DeleteBookmark(LPCTSTR lpstrBookmarkName)
{
	int nPos=IndexOfBookmark(lpstrBookmarkName);
	if(nPos!=-1)
	{
		m_arrayBookmarks.RemoveAt(nPos);
		SetModifiedFlag();
		return TRUE;
	}

	return FALSE;
}

	

BOOL CDrawDoc::Navigate(LPCTSTR lpstrBookmarkName,CDrawView* pView)
{
	if(lpstrBookmarkName==NULL)
		return TRUE;

	int nPos=IndexOfBookmark(lpstrBookmarkName);
	if(nPos!=-1)
	{
		pView->NavigateToBookmark(m_arrayBookmarks[nPos]);
		return TRUE;
	}

#ifndef _UNICODE
#define _tcscanf sscanf
#else
#define _tcscanf swscanf
#endif

	int nXpos,nYpos;
	if(_tcscanf(lpstrBookmarkName,_T("Bookmark %d:%d"),&nXpos,&nYpos)==2)
	{
		Bookmark bm;
		bm.m_ptBookmark.x=nXpos;
		bm.m_ptBookmark.y=nYpos;

		pView->NavigateToBookmark(bm);
		return TRUE;
	}

	return FALSE;
}

int CDrawDoc::IndexOfBookmark(LPCTSTR lpstrBookmarkName)
{
	if(lpstrBookmarkName)
	{
		for(int i=0; i<m_arrayBookmarks.GetSize(); i++)
		{
			if(m_arrayBookmarks[i].m_strName==lpstrBookmarkName)
				return i;
		}
	}

	return -1;
}



CString CDrawDoc::CreateOrGetBookmarkAtCurrentPos(CDrawView* pView)
{
	CPoint pos;

	if(pView)
		pos=pView->GetScrollPosition();
	
	for(int i=0; i<m_arrayBookmarks.GetSize(); i++)
	{
		Bookmark bm=m_arrayBookmarks[i];
		if(bm.m_ptBookmark==pos)
			return bm.m_strName;
	}

	CString strNewBookmarkName;
	strNewBookmarkName.Format(_T("Bookmark %d:%d"),pos.x,pos.y);

	//We do not create bookmark since our code can naigate to
	//strings like Bookmark 10:10

	return strNewBookmarkName;
}


//Bookmarks end


//HLink begin
DWORD CDrawDoc::GetNextSiteData() const
{
	DWORD dwNext=0;
	POSITION pos=m_objects.GetHeadPosition();
	while(pos)
	{
		CDrawObj* pObj=m_objects.GetNext(pos);
		if(pObj->IsKindOf(RUNTIME_CLASS(CDrawHLink)))
		{
			CDrawHLink* pLink=(CDrawHLink*)pObj;
			dwNext=max(pLink->m_dwSiteData,dwNext);
		}
	}

	return ++dwNext;
}


IHlinkPtr CDrawDoc::CreateHlink(LPCTSTR szTarget, LPCTSTR szLocation, LPCTSTR szFriendly,DWORD dwSiteData)
{
	USES_CONVERSION;

	HRESULT hr;

	IHlinkPtr pHlink;
	hr = HlinkCreateFromString(T2OLE(szTarget),
								T2OLE(szLocation),
								T2OLE(szFriendly),
								&m_xHlinkSite,
								dwSiteData,
								NULL,
								IID_IHlink,
								(void**) &pHlink);

	if (SUCCEEDED(hr))
	{
		pHlink->SetHlinkSite(&m_xHlinkSite,dwSiteData);
	}

	return pHlink;
}


IHlinkPtr CDrawDoc::CreateHlinkFromData(COleDataObject* pDataObject,DWORD dwSiteData)
{
	HRESULT hr;

	IHlinkPtr pHlink;
	hr = HlinkCreateFromData(pDataObject->Detach(),
								&m_xHlinkSite,
								dwSiteData,
								NULL,
								IID_IHlink,
								(void**)&pHlink);
	if(SUCCEEDED(hr))
	{
		pHlink->SetHlinkSite(&m_xHlinkSite,dwSiteData);
	}

	return pHlink;
}


CDrawView* CDrawDoc::GetViewForNavigate()
{
	POSITION pos=GetFirstViewPosition();

	while(pos)
	{
		CView* pView=GetNextView(pos);
		if(pView->IsKindOf(RUNTIME_CLASS(CDrawView)))
		{
			CDrawView* pDrawView=(CDrawView*)pView;
			if(pDrawView==m_pLastActivatedView)
				return pDrawView;
		}
	}
	
	pos=GetFirstViewPosition();
	while(pos)
	{
		CView* pView=GetNextView(pos);
		if(pView->IsKindOf(RUNTIME_CLASS(CDrawView)))
		{
			CDrawView* pDrawView=(CDrawView*)pView;
			return pDrawView;
		}
	}
	
	
	return NULL;
}


void CDrawDoc::OnActivateView(CDrawView* pView, BOOL bActive)
{
	if(bActive)
		m_pLastActivatedView=pView;
	else if(m_pLastActivatedView==pView)
		m_pLastActivatedView=NULL;
}



IHlinkFramePtr CDrawDoc::GetHlinkFrame()
{
	IHlinkFramePtr phlFrame;

	// if this hyperlink target is an Active Document object, try to
    // retrieve the hyperlink frame pointer from the IOleInPlaceFrame
    if (m_lpClientSite)
	{
		IOleInPlaceSitePtr pInPlaceSite;
		if(SUCCEEDED(m_lpClientSite->QueryInterface(IID_IOleInPlaceSite,(void**)&pInPlaceSite)))
		{
			IOleInPlaceFramePtr pFrame;
			IOleInPlaceUIWindowPtr pDoc;
			RECT rcPosRect,rcClipRect;
			OLEINPLACEFRAMEINFO FrameInfo;
			FrameInfo.cb=sizeof(OLEINPLACEFRAMEINFO);

			if(SUCCEEDED(pInPlaceSite->GetWindowContext(&pFrame,&pDoc,&rcPosRect,&rcClipRect,&FrameInfo)))
			{
				if(pFrame)
					pFrame->QueryInterface(IID_IHlinkFrame, (void**)&phlFrame);
			}
		}

		if(phlFrame==NULL)
		{
			IServiceProviderPtr pSP;
			if(SUCCEEDED(m_lpClientSite->QueryInterface(IID_IServiceProvider,(void**)&pSP)))
				pSP->QueryService(SID_SHlinkFrame, IID_IHlinkFrame,(void**)&phlFrame);
		}
	}

	return phlFrame;
}


void CDrawDoc::AddCurrentLocationToBrowseContext(CDrawView* pView)
{
	USES_CONVERSION;

	IHlinkBrowseContextPtr pHBC=GetBrowseContext();

	CString strBookmarkName=CreateOrGetBookmarkAtCurrentPos(pView);
	LPCWSTR lpcwstrBookmark=T2OLE(strBookmarkName);
	
	IMonikerPtr pMoniker;
	pMoniker.Attach(GetMoniker(OLEGETMONIKER_FORCEASSIGN));
	pHBC->OnNavigateHlink(0, pMoniker, lpcwstrBookmark, lpcwstrBookmark,NULL);
}


BOOL CDrawDoc::SetCurrentLocationToClipboard(CDrawView* pView)
{
	USES_CONVERSION;
	CString strBookmarkName=CreateOrGetBookmarkAtCurrentPos(pView);
	

	IMonikerPtr pMoniker;
	pMoniker.Attach(GetMoniker(OLEGETMONIKER_FORCEASSIGN));
	
	IHlinkPtr pHlink;
	HRESULT hr;
	hr = HlinkCreateFromMoniker(pMoniker,
								T2OLE(strBookmarkName),
								T2OLE(strBookmarkName),
								NULL,
								0,
								NULL,
								IID_IHlink,
								(void**) &pHlink);

	if(SUCCEEDED(hr))
	{
		IDataObjectPtr pDataObject=pHlink;
		::OleSetClipboard(pDataObject);
		return TRUE;
	}

	return FALSE;
}


IHlinkBrowseContextPtr CDrawDoc::GetBrowseContext()
{
	if(m_pIHlinkBrowseContext==NULL)
	{
		IHlinkFramePtr phlFrame=GetHlinkFrame();
		if (phlFrame!=NULL)
		    phlFrame->GetBrowseContext(&m_pIHlinkBrowseContext);

		if(m_pIHlinkBrowseContext==NULL)
			HlinkCreateBrowseContext(NULL,IID_IHlinkBrowseContext,(void**)&m_pIHlinkBrowseContext);

		if (m_pIHlinkBrowseContext != NULL)
		{	
			// register with the browse context
			IMonikerPtr pMoniker;
			pMoniker.Attach(GetMoniker(OLEGETMONIKER_FORCEASSIGN));
			m_pIHlinkBrowseContext->Register(0,
											GetInterface((void*)&IID_IUnknown), 
											pMoniker, 
											&m_dwBrowseContextRegister);

			// because we want the user to be able to come back to this
			// object via GoBack functionality add self to the 
			// navigation stack. This effectively tells the browse context
			// that the current hyperlink container referred to by pMoniker 
			// is a member (at the top) of the navigation stack
			AddCurrentLocationToBrowseContext(GetViewForNavigate());
		}
		else
			abort();
	}

	return m_pIHlinkBrowseContext;
}


void CDrawDoc::HideAfterNavigation(CDrawView* pView,BOOL bHide)
{
	CMainFrame* pMainFrame=GetMainFrame();
	if(bHide)
	{
		if(pMainFrame->m_bHidePending)
			pMainFrame->ShowWindow(SW_MINIMIZE);

		if(m_bHidePending)
		{
			//::SetWindowPos(pView->GetParentFrame()->GetSafeHwnd(),HWND_BOTTOM,0,0,0,0,SWP_NOMOVE|SWP_NOSIZE|SWP_NOACTIVATE);
			pView->GetParentFrame()->ActivateFrame(SW_HIDE);
		}
	}
	else
	{
		m_bHidePending = FALSE;
		pMainFrame->m_bHidePending=FALSE;
	}
}

HRESULT CDrawDoc::NavigateToHLink(IHlinkPtr pHlink,DWORD grfHLNF,CDrawView* pView)
{
	if(GetPathName().IsEmpty())
	{
		AfxMessageBox(_T("Can't navigate until file is saved"));
		return E_FAIL;
	}

	IHlinkBrowseContextPtr pHBC=GetBrowseContext();
	IHlinkFramePtr phlFrame=GetHlinkFrame();
	
	CMainFrame* pMainFrame=GetMainFrame();
	CFrameWnd* pMDIChildFrame=pView->GetParentFrame();

	if (!IsDocObject())
	{
		HLBWINFO hlbwinfo;
        // initialize hlbwinfo with window locations and flags
        // register the browse window info in the browse context 
        // so it is later available to the hyperlink target and
		memset(&hlbwinfo,0,sizeof(HLBWINFO));
		hlbwinfo.cbSize=sizeof(HLBWINFO);
		hlbwinfo.grfHLBWIF=HLBWIF_HASFRAMEWNDINFO|HLBWIF_HASDOCWNDINFO;

		WINDOWPLACEMENT wp;
		pMainFrame->GetWindowPlacement(&wp);

		if(wp.showCmd==SW_SHOWMAXIMIZED)
			hlbwinfo.grfHLBWIF|=HLBWIF_FRAMEWNDMAXIMIZED;

		CopyRect(&hlbwinfo.rcFramePos,&wp.rcNormalPosition);

		
		
		pMDIChildFrame->GetWindowPlacement(&wp);
		
		if(wp.showCmd==SW_SHOWMAXIMIZED)
			hlbwinfo.grfHLBWIF|=HLBWIF_DOCWNDMAXIMIZED;

		CopyRect(&hlbwinfo.rcDocPos,&wp.rcNormalPosition);
		
		pHBC->SetBrowseWindowInfo(&hlbwinfo);
	}


	IBindCtxPtr pBindCtx;
	CreateBindCtx(  0, &pBindCtx );
    
	m_bHidePending = !IsDocObject();
	pMainFrame->m_bHidePending=m_bHidePending;

	HRESULT hr = HlinkNavigate(pHlink, 
								phlFrame,
								grfHLNF,
								pBindCtx,
								NULL,
								pHBC);
		
	HideAfterNavigation(pView,SUCCEEDED(hr));
	return hr;
}


HRESULT CDrawDoc::NavigateToDrawHLink(CDrawHLink* pDrawHLink,CDrawView* pView)
{
	if(pDrawHLink->m_pHlink==NULL)
		return E_FAIL;

	return NavigateToHLink(pDrawHLink->m_pHlink,0,pView);
}


void CDrawDoc::HlinkGoforward(CDrawView* pView) 
{
	// TODO: Add your command handler code here
	IHlinkBrowseContextPtr pHBC=GetBrowseContext();
	
	IHlinkPtr pHlink;
	if(FAILED(pHBC->GetHlink(HLID_NEXT,&pHlink)))
		return;

	NavigateToHLink(pHlink,HLNF_NAVIGATINGFORWARD,pView);
}


BOOL CDrawDoc::CanGoforward() 
{
	// TODO: Add your command update UI handler code here
	BOOL bEnable=FALSE;
	if(m_pIHlinkBrowseContext)
	{
		if(m_pIHlinkBrowseContext->QueryHlink(HLQF_ISVALID,HLID_NEXT)==S_OK)
			bEnable=TRUE;
	}

	return bEnable;
}


void CDrawDoc::HlinkGoback(CDrawView* pView) 
{
	// TODO: Add your command handler code here
	IHlinkBrowseContextPtr pHBC=GetBrowseContext();

	IHlinkPtr pHlink;
	if(FAILED(pHBC->GetHlink(HLID_PREVIOUS,&pHlink)))
		return;


	NavigateToHLink(pHlink,HLNF_NAVIGATINGBACK,pView);
}


BOOL CDrawDoc::CanGoback() 
{
	// TODO: Add your command update UI handler code here
	BOOL bEnable=FALSE;
	if(m_pIHlinkBrowseContext)
	{
		if(m_pIHlinkBrowseContext->QueryHlink(HLQF_ISVALID,HLID_PREVIOUS)==S_OK)
			bEnable=TRUE;
	}

	return bEnable;
}



STDMETHODIMP_(ULONG) CDrawDoc::XHlinkTarget::AddRef()
{
	METHOD_PROLOGUE_EX_(CDrawDoc , HlinkTarget)
	return (ULONG)pThis->ExternalAddRef();
}

STDMETHODIMP_(ULONG) CDrawDoc ::XHlinkTarget::Release()
{
	METHOD_PROLOGUE_EX_(CDrawDoc , HlinkTarget)
	return (ULONG)pThis->ExternalRelease();
}

STDMETHODIMP CDrawDoc::XHlinkTarget::QueryInterface(
	REFIID iid, LPVOID* ppvObj)
{
	METHOD_PROLOGUE_EX_(CDrawDoc, HlinkTarget)
	return (HRESULT)pThis->ExternalQueryInterface(&iid, ppvObj);
}


STDMETHODIMP CDrawDoc::XHlinkTarget::SetBrowseContext(IHlinkBrowseContext __RPC_FAR *pihlbc)
{
	METHOD_PROLOGUE_EX_(CDrawDoc, HlinkTarget)
	
	if(pThis->m_pIHlinkBrowseContext!= NULL)
        pThis->m_pIHlinkBrowseContext->Revoke(pThis->m_dwBrowseContextRegister);
    
	pThis->m_pIHlinkBrowseContext=pihlbc;

    if(pihlbc!=NULL)
	{
		IMonikerPtr pMoniker;
		pMoniker.Attach(pThis->GetMoniker(OLEGETMONIKER_FORCEASSIGN));
		pihlbc->Register(0, this, pMoniker, &pThis->m_dwBrowseContextRegister);
	}

	return S_OK;
}


STDMETHODIMP CDrawDoc::XHlinkTarget::GetBrowseContext(IHlinkBrowseContext __RPC_FAR *__RPC_FAR *ppihlbc)
{
	METHOD_PROLOGUE_EX_(CDrawDoc, HlinkTarget)

	//Here we do no call GetBrowseContext to avoid allocation
	//of the new context in case of absence of the one
	*ppihlbc=pThis->m_pIHlinkBrowseContext;
	if(*ppihlbc)
	{
		(*ppihlbc)->AddRef();
		return S_OK;
	}
	
	return E_FAIL;
}




STDMETHODIMP CDrawDoc::XHlinkTarget::Navigate(DWORD grfHLNF,LPCWSTR pwzJumpLocation)
{
	USES_CONVERSION;

	METHOD_PROLOGUE_EX_(CDrawDoc, HlinkTarget)

	CDocObjectServerSpy* pDocObjServer=(CDocObjectServerSpy*)pThis->m_pDocObjectServer;
	if(pDocObjServer)
	{
		if(!pDocObjServer->IsActive())
			pDocObjServer->ActivateDocObject();
	}
	else
		AfxOleSetUserCtrl(true);

	CDrawView* pView=pThis->GetViewForNavigate();
	if(!pView)
		return E_FAIL;

	IHlinkBrowseContextPtr pHBC=pThis->GetBrowseContext();
	IHlinkFramePtr phlFrame=pThis->GetHlinkFrame();

	// notify the hlink frame and the browse context that the navigation
    // is complete.
	IMonikerPtr pMoniker;
	pMoniker.Attach(pThis->GetMoniker(OLEGETMONIKER_FORCEASSIGN));
	HlinkOnNavigate(phlFrame, pHBC, grfHLNF, pMoniker, pwzJumpLocation, pwzJumpLocation,NULL);

	if (phlFrame == NULL) 
	{
		if(!pThis->IsDocObject() && !(grfHLNF & HLNF_INTERNALJUMP))
		{
			CMainFrame* pMainFrame=GetMainFrame();

			int nMainCmdShow=SW_SHOWNORMAL;
			int nDocCmdShow=SW_SHOWNORMAL;

			CRect rectMainPos,rectDocPos;

			HLBWINFO hlbwi;
			hlbwi.cbSize=sizeof(HLBWINFO);
			if(SUCCEEDED(pHBC->GetBrowseWindowInfo(&hlbwi)))
			{
				// adjust the document and frame windows according 
				// to the dimensions in HLBWI
				if(hlbwi.grfHLBWIF & HLBWIF_HASFRAMEWNDINFO)
				{
					if(hlbwi.grfHLBWIF & HLBWIF_FRAMEWNDMAXIMIZED)
						nMainCmdShow=SW_SHOWMAXIMIZED;
					CopyRect(&rectMainPos,&hlbwi.rcFramePos);
				}
				
				if(hlbwi.grfHLBWIF & HLBWIF_HASDOCWNDINFO && pView)
				{
					CopyRect(&rectDocPos,&hlbwi.rcDocPos);
					
					if(hlbwi.grfHLBWIF & HLBWIF_DOCWNDMAXIMIZED)
						nDocCmdShow=SW_SHOWMAXIMIZED;
				}
			}

			//Ensure our view is visible
			pMainFrame->ActivateFrame(nMainCmdShow);
			if(nMainCmdShow!=SW_SHOWMAXIMIZED)
			{
				if(!rectMainPos.IsRectEmpty())
					pMainFrame->MoveWindow(rectMainPos);
			}

			
			CFrameWnd* pDocFrame=pView->GetParentFrame();

			pDocFrame->ActivateFrame(nDocCmdShow);
			if(nDocCmdShow!=SW_SHOWMAXIMIZED)
			{
				if(!rectDocPos.IsRectEmpty())
					pDocFrame->MoveWindow(rectDocPos);
			}
					
			
			pDocFrame->SetActiveView(pView);
		}
	}

	
	pThis->Navigate(OLE2T(pwzJumpLocation),pView);
		
	pThis->m_bHidePending=FALSE;
	GetMainFrame()->m_bHidePending=FALSE;
	
	return S_OK;
}


STDMETHODIMP CDrawDoc::XHlinkTarget::GetMoniker(LPCWSTR pwzLocation,DWORD dwAssign,IMoniker __RPC_FAR *__RPC_FAR *ppimkLocation)
{
	USES_CONVERSION;
	METHOD_PROLOGUE_EX_(CDrawDoc, HlinkTarget)
	
	*ppimkLocation=pThis->GetMoniker((OLEGETMONIKER)dwAssign);
	if(*ppimkLocation==NULL)
		return E_FAIL;
	
	return S_OK;
}


STDMETHODIMP CDrawDoc::XHlinkTarget::GetFriendlyName(LPCWSTR pwzLocation,LPWSTR __RPC_FAR *ppwzFriendlyName)
{
	USES_CONVERSION;
	METHOD_PROLOGUE_EX_(CDrawDoc, HlinkTarget)

	int nPos=pThis->IndexOfBookmark(OLE2T(pwzLocation));
	if(nPos!=-1)
	{
		*ppwzFriendlyName=(LPWSTR)CoTaskMemAlloc(sizeof(WCHAR)*(wcslen(pwzLocation)+1));
		wcscpy(*ppwzFriendlyName,pwzLocation);
		return S_OK;
	}

	return E_FAIL;
}





STDMETHODIMP_(ULONG) CDrawDoc::XHlinkSite::AddRef()
{
	METHOD_PROLOGUE_EX_(CDrawDoc , HlinkSite)
	return (ULONG)pThis->ExternalAddRef();
}

STDMETHODIMP_(ULONG) CDrawDoc ::XHlinkSite::Release()
{
	METHOD_PROLOGUE_EX_(CDrawDoc , HlinkSite)
	return (ULONG)pThis->ExternalRelease();
}

STDMETHODIMP CDrawDoc::XHlinkSite::QueryInterface(
	REFIID iid, LPVOID* ppvObj)
{
	METHOD_PROLOGUE_EX_(CDrawDoc, HlinkSite)
	return (HRESULT)pThis->ExternalQueryInterface(&iid, ppvObj);
}

STDMETHODIMP CDrawDoc::XHlinkSite::QueryService(DWORD dwSiteData,REFGUID guidService,REFIID riid,IUnknown __RPC_FAR *__RPC_FAR *ppiunk)
{
	METHOD_PROLOGUE_EX_(CDrawDoc, HlinkSite)
	if(IsEqualGUID(riid,IID_IHlinkTarget))
	{
		*ppiunk=&pThis->m_xHlinkTarget;
		pThis->m_xHlinkTarget.AddRef();
		return S_OK;
	}

	return E_NOINTERFACE;
}

STDMETHODIMP CDrawDoc::XHlinkSite::GetMoniker(DWORD dwSiteData,DWORD dwAssign,DWORD dwWhich,IMoniker __RPC_FAR *__RPC_FAR *ppimk)
{
	METHOD_PROLOGUE_EX_(CDrawDoc, HlinkSite)
	if(dwWhich==OLEWHICHMK_CONTAINER)
		*ppimk=pThis->GetMoniker((OLEGETMONIKER)dwAssign);
	return S_OK;
}

STDMETHODIMP CDrawDoc::XHlinkSite::ReadyToNavigate(DWORD dwSiteData,DWORD dwReserved)
{
	METHOD_PROLOGUE_EX_(CDrawDoc, HlinkSite)
	return S_OK;
}

STDMETHODIMP CDrawDoc::XHlinkSite::OnNavigationComplete(DWORD dwSiteData,DWORD dwreserved,HRESULT hrError,LPCWSTR pwzError)
{
	USES_CONVERSION;
	METHOD_PROLOGUE_EX_(CDrawDoc, HlinkSite)

	if(FAILED(hrError))
	{
		CString strMessage(_T("Failed to navigate"));
		strMessage+=_T("\n\n");
		strMessage+=OLE2T(pwzError);

		AfxMessageBox(strMessage);
	}
	
	return S_OK;
}

//HLink end

