// drawdoc.h : interface of the CDrawDoc class
//
// This is a part of the Microsoft Foundation Classes C++ library.
// Copyright (C) 1992-1998 Microsoft Corporation
// All rights reserved.
//
// This source code is only intended as a supplement to the
// Microsoft Foundation Classes Reference and related
// electronic documentation provided with the library.
// See these sources for detailed information regarding the
// Microsoft Foundation Classes product.

#include "drawobj.h"
#include "summinfo.h"

class CDrawView;
class CDrawSrvrItem;


class CDrawDoc : public COleServerDoc //Doc server
{
protected: // create from serialization only
	CDrawDoc();
	DECLARE_DYNCREATE(CDrawDoc)

// Attributes
public:
	CDrawObjList* GetObjects() { return &m_objects; }
	const CSize& GetSize() const { return m_size; }
	void ComputePageSize();
	int GetMapMode() const { return m_nMapMode; }
	COLORREF GetPaperColor() const { return m_paperColor; }
	CSummInfo *m_pSummInfo;

	//Bookmarks begin
	class Bookmark
	{
	public:
		CString m_strName;
		CPoint	m_ptBookmark;
		BOOL	m_bHidden;

		Bookmark();
		Bookmark(const Bookmark& other);
		Bookmark& operator=(const Bookmark& other);

		void Serialize(CArchive& ar);
	};

	
	class BookmarkArray : public CArray<Bookmark,Bookmark&>
	{
	};

	const BookmarkArray& GetBookmarks() const
	{
		return m_arrayBookmarks;
	}
	//Bookmarks end


	
// Operations
public:
	CDrawObj* ObjectAt(const CPoint& point);
	void Draw(CDC* pDC, CDrawView* pView);
	void Add(CDrawObj* pObj);
	void Remove(CDrawObj* pObj);

	//Bookmarks begin
	BOOL CreateBookmark(LPCTSTR lpstrBookmarkName,CPoint ptDoc,BOOL bHidden=FALSE);
	BOOL DeleteBookmark(LPCTSTR lpstrBookmarkName);
	CString CreateOrGetBookmarkAtCurrentPos(CDrawView* pView);
	
	
	BOOL Navigate(LPCTSTR lpstrBookmarkName,CDrawView* pView);
	//Bookmarks end

	//Hlink begin
	void HlinkGoforward(CDrawView* pView);
	BOOL CanGoforward();
	void HlinkGoback(CDrawView* pView);
	BOOL CanGoback();

	void OnActivateView(CDrawView* pView, BOOL bActive);
	CDrawView* GetViewForNavigate();
	BOOL SetCurrentLocationToClipboard(CDrawView* pView);
	HRESULT NavigateToDrawHLink(CDrawHLink* pDrawHLink,CDrawView* pView);
	//Hlink end

// Implementation
public:
	virtual ~CDrawDoc();
	virtual void Serialize(CArchive& ar);   // overridden for document i/o
	virtual COleServerItem* OnGetEmbeddedItem(); //Doc server
	
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
	virtual BOOL OnNewDocument();
	virtual BOOL OnOpenDocument(LPCTSTR lpszPathName);
	virtual BOOL OnSaveDocument(LPCTSTR lpszPathName);
	virtual void OnCloseDocument(); //Hlink

	//Doc server begin
	virtual CDocObjectServer* GetDocObjectServer(LPOLEDOCUMENTSITE pDocSite);
	//Doc server end

	

	CDrawObjList m_objects;
	CSize m_size;
	int m_nMapMode;
	COLORREF m_paperColor;

	//Bookmarks begin
	BookmarkArray m_arrayBookmarks;
	int IndexOfBookmark(LPCTSTR lpstrBookmarkName);

	CDrawView*	m_pLastActivatedView;
	//Bookmarks end

// Generated message map functions
protected:
	//{{AFX_MSG(CDrawDoc)
	afx_msg void OnViewPaperColor();
	afx_msg void OnFileSummaryInfo();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()

	//HLink begin
	friend class CDrawHLink;

	IHlinkBrowseContextPtr	m_pIHlinkBrowseContext;
	DWORD					m_dwBrowseContextRegister;
	BOOL					m_bHidePending;

	IHlinkFramePtr GetHlinkFrame();
	IHlinkBrowseContextPtr GetBrowseContext();
	void AddCurrentLocationToBrowseContext(CDrawView* pView);
	void HideAfterNavigation(CDrawView* pView,BOOL bHide);
	DWORD GetNextSiteData() const;
	IHlinkPtr CreateHlink(LPCTSTR szTarget, LPCTSTR szLocation, LPCTSTR szFriendly,DWORD dwSiteData);
	IHlinkPtr CreateHlinkFromData(COleDataObject* pDataObject,DWORD dwSiteData);
	HRESULT NavigateToHLink(IHlinkPtr pHlink,DWORD grfHLNF,CDrawView* pView);
	

	BEGIN_INTERFACE_PART(HlinkTarget, IHlinkTarget)
		INIT_INTERFACE_PART(CDrawDoc, HlinkTarget)
		STDMETHOD(SetBrowseContext)(IHlinkBrowseContext __RPC_FAR *pihlbc);
        STDMETHOD(GetBrowseContext)(IHlinkBrowseContext __RPC_FAR *__RPC_FAR *ppihlbc);
        STDMETHOD(Navigate)(DWORD grfHLNF,LPCWSTR pwzJumpLocation);
        STDMETHOD(GetMoniker)(LPCWSTR pwzLocation,DWORD dwAssign,IMoniker __RPC_FAR *__RPC_FAR *ppimkLocation);
        STDMETHOD(GetFriendlyName)(LPCWSTR pwzLocation,LPWSTR __RPC_FAR *ppwzFriendlyName);
	END_INTERFACE_PART(HlinkTarget)

	BEGIN_INTERFACE_PART(HlinkSite, IHlinkSite)
		INIT_INTERFACE_PART(CDrawDoc, HlinkSite)
		STDMETHOD(QueryService)(DWORD dwSiteData,REFGUID guidService,REFIID riid,IUnknown __RPC_FAR *__RPC_FAR *ppiunk);
        STDMETHOD(GetMoniker)(DWORD dwSiteData,DWORD dwAssign,DWORD dwWhich,IMoniker __RPC_FAR *__RPC_FAR *ppimk);
        STDMETHOD(ReadyToNavigate)(DWORD dwSiteData,DWORD dwReserved);
        STDMETHOD(OnNavigationComplete)(DWORD dwSiteData,DWORD dwreserved,HRESULT hrError,LPCWSTR pwzError);
	END_INTERFACE_PART(HlinkSite)

	DECLARE_INTERFACE_MAP()
	//HLink end
};

class CDocObjectServerSpy : public CDocObjectServer
{
public:
	BOOL IsActive() const
	{
		return m_pViewSite!=NULL;
	}
};

/////////////////////////////////////////////////////////////////////////////
